{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">                      
            <li class="active">{{ _('Home') }}</li>                
          </ul>                                                                    
          <br />
          <h1 class="page-header">{{ _('Servers') }}</h1>

          <div class="table-responsive">
            <table class="table table-striped table-hover nowrap" id="table-servers" width="100%">
              <thead>
                <tr>
                  <th>{{ _('Name') }}</th>
                  <th class="desktop">{{ _('Clients') }}</th>
                  <th class="desktop">{{ _('Status') }}</th>
                </tr>
              </thead>
              <tbody>
              </tbody>
            </table>
          </div>
        </div>
{% endblock %}
