{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">
            <li><a href="{{ url_for('view.home') }}">{{ _('Home') }}</a></li>
            <li class="active">{{ _('Administration') }}</li>
          </ul>
          <br />
          <div class="form-container">
            <legend>{{ _('Authentication Backends list') }}</legend>
            <div class="table-responsive" id="table-auth-backends-container">
              <table class="table table-striped table-hover nowrap" id="table-auth-backends" width="100%">
                <thead>
                  <tr>
                    <th>{{ _('Backend') }}</th>
                    <th>{{ _('Description') }}</th>
                    <th>{{ _('Priority') }}</th>
                    <th>{{ _('Type') }}</th>
                  </tr>
                </thead>
                <tbody>
                </tbody>
              </table>
            </div>
          </div>
          <div class="form-container">
            <legend>{{ _('Authorization Backends list') }}</legend>
            <div class="table-responsive" id="table-backends-container">
              <table class="table table-striped table-hover nowrap" id="table-backends" width="100%">
                <thead>
                  <tr>
                    <th>{{ _('Backend') }}</th>
                    <th>{{ _('Description') }}</th>
                    <th>{{ _('Priority') }}</th>
                    <th>{{ _('Type') }}</th>
                  </tr>
                </thead>
                <tbody>
                </tbody>
              </table>
            </div>
          </div>
        </div>
{% endblock %}
